package lc.esp.hello.service;

import lc.esp.sdk.*;
import lc.mecha.json.JSONObject;
import lc.mecha.log.MechaLogger;
import lc.mecha.log.MechaLoggerFactory;
import lc.mecha.util.BasicallyDangerous;

/**
 * The hello service greets a caller. This service is really only useful for dev/test and as an illustrative example
 * of the request/response cycle in ESP18.
 *
 * @author Alex Leigh
 * @since mk18 (GIPSY DANGER)
 */
public class HelloService extends BasicallyDangerous {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(HelloService.class);

    public static void main(String[] args) {
        new HelloService().run();
    }

    @Override
    public void runDangerously() throws Exception {
        try (ESPClient esp = new ESPClient().start()) {
            try (ESPSession session = esp.createSession()) {
                ESPAddress dst = new ESPAddress("leighco", "devtest", "example", "hello",
                        ESPAddressClass.QUEUE, ESPMessageClass.COMMAND);
                try (ESPConsumer consumer = session.createConsumer(dst)) {
                    //noinspection InfiniteLoopStatement
                    while (true) {
                        ESPMessage request = consumer.receive();
                        ESPAddress replyAddress = request.getReplyTo();
                        if (replyAddress != null) {
                            try (ESPProducer producer = session.createProducer(replyAddress)) {
                                producer.send(buildReply(request));
                            }
                        } else {
                            logger.warn("Requestor did not supply a return address: {}", request);
                        }
                    }
                }
            }
        }
    }

    private ESPMessage buildReply(ESPMessage request) {
        String theirName = request.getPayload().getString("name");
        JSONObject replyJson = new JSONObject();
        replyJson.put("msg", "Hello, " + theirName);
        return new ESPMessage(replyJson, null, request.getCorrelationId());
    }
}